#include "./generator-rf.h"
#include <random>
#include <cmath>
#include <stdexcept>
namespace mri {
  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  void GeneratorRF::EvaluateSignal(std::vector<double> &signal_rf) {
    signal_rf.resize(total_samples_);
    for (long i = 0; i < total_samples_; ++i) {
      signal_rf[i] = 0.0;
      double time = timesteps_[i];
      for (unsigned int voxel = 0; voxel < voxel_num_; ++voxel) {
        signal_rf[i] +=
          amplitudes_[voxel]
          * std::sin(
                     omega_*time + phases_[voxel]
                     )
          * std::exp(-time/
                     (T2s_decays_norm_[voxel]*T2s_scale_
                      )
                     )
          // ) + ( 0.0 
          //                                           + np.random.rand()/noise_ratio

          ;
      }
    }
  };
  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  void GeneratorRF::RandomizeVoxels(){
    std::random_device rd;
    std::mt19937_64 om;
    om.seed(rd());
    double lbound = 0.0, ubound = 1.0;
    std::uniform_real_distribution<double> rand(lbound, ubound);
    amplitudes_.clear();
    T2s_decays_norm_.clear();
    amplitudes_.resize(voxel_num_);
    T2s_decays_norm_.resize(voxel_num_);
    for (double &amp:amplitudes_){
      amp = rand(om);
    }
    for (double &decay:T2s_decays_norm_){
      decay = rand(om);
    }
  };

  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  void GeneratorRF::SetAmplitudes(std::vector<double> amplitudes){
    if (amplitudes.size() != voxel_num_)
      throw std::invalid_argument("Number of amplitudes do not fit number of voxels");    
    amplitudes_ = amplitudes;
  };
  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  void GeneratorRF::SetT2sDecaysNorm(std::vector<double> T2s_decays_norm){
    T2s_decays_norm_ = T2s_decays_norm;
  };
  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  // period is set in milliseconds (ms)
  void GeneratorRF::SetTimer(long total_periods, double period_ms,
                             int samples_per_period) {
    total_periods_ = total_periods;
    period_ms_ = period_ms;
    samples_per_period_ = samples_per_period;
    omega_ = 2.0*PI_/period_ms_;
    total_samples_ = total_periods * samples_per_period;
    timesteps_.resize(total_samples_);
    double a_timestep = period_ms/samples_per_period;
    for (long i = 0; i < total_samples_; ++i) {
      timesteps_[i] = i*a_timestep;      
    }
    // for (double step: timesteps_) std::cout << step<<' ';
    // std::cout<<std::endl;
  };

  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //
  void GeneratorRF::SetVoxels(double voxel_num, double phase_init,
                              double phase_range, double T2s_scale) {
    T2s_scale_ = T2s_scale;
    if (voxel_num < 2)
      throw std::invalid_argument("We need at least two voxels for phase gradient");    
    voxel_num_ = voxel_num;
    phase_init_ = phase_init;
    phase_range_ = phase_range;
    double phase_step = phase_range_/(voxel_num_ - 1);
    phases_.resize(voxel_num);
    for (unsigned int i =0; i< voxel_num_; ++i) {
      phases_[i] = phase_init_ + i*phase_step;
    }
  };
  // ********************************************************************** //
  // ********************************************************************** //
  // ********************************************************************** //

}  // end of namespace mri
